use std::path::Path;

use actix_files::Files;
use actix_web::{middleware::Logger, web, App, HttpResponse, HttpServer};
use clap::Parser;

/// Music web player server
#[derive(Parser, Debug, Clone)]
#[clap(author, version, about, long_about = None)]
struct Args {
    /// Listen address
    #[clap(short, long, env, default_value = "0.0.0.0:8000")]
    listen_address: String,

    /// Path to files
    #[clap(short, long)]
    files_path: String,

    /// Api URL
    #[clap(long, env)]
    api_url: String,

    /// Api token
    #[clap(long, env)]
    api_token: String,
}

/// Serve main file, modifying api url & token values
async fn main_js_file(data: web::Data<Args>) -> HttpResponse {
    let path = Path::new(&data.files_path).join("main.dart.js");
    let file = std::fs::read_to_string(path).unwrap();
    let script = file
        .replace("<<<API_URL>>>", &data.api_url)
        .replace("<<<API_TOKEN>>>", &data.api_token)
        .replace(
            "https://fonts.gstatic.com/s/roboto/v20/KFOmCnqEu92Fr1Me5WZLCzYlKw.ttf",
            "/fonts/KFOmCnqEu92Fr1Me5WZLCzYlKw.ttf",
        );

    HttpResponse::Ok()
        .content_type("application/javascript")
        .body(script)
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    let args: Args = Args::parse();

    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));
    log::info!("starting HTTP server at {}", args.listen_address);

    let args_cp = args.clone();
    HttpServer::new(move || {
        App::new()
            .service(web::resource("main.dart.js").to(main_js_file))
            .service(Files::new("/", &args_cp.files_path).index_file("index.html"))
            .wrap(Logger::default())
            .app_data(web::Data::new(args_cp.clone()))
    })
    .bind(args.listen_address)?
    .run()
    .await
}
